@@ -14,18 +14,6 @@ else
     exit 1
 fi
 
-##
-## what version do we need?
-##
-if [ `$AUTOCONF --version | head -1 | cut -d.  -f 2` -lt 53 ]; then
-
-	## maybe it's installed under a different name (e.g. RedHat 7.3)
-
-	AUTOCONF="autoconf-2.53"
-	AUTOHEADER="autoheader-2.53"
-
-fi
-
 echo "$0: running $AUTOHEADER"
 $AUTOHEADER || exit 1
 
