@@ -1,9 +1,8 @@
-/* -*- c-file-style: "java"; indent-tabs-mode: nil -*-
+/* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78; -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/bulk.h,v 1.1 2002/08/31 22:32:20 mbp Exp $ 
  *
- * Copyright (C) 2002 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +20,20 @@
  * USA
  */
 
-int dcc_r_file(int ifd, const char *filename, size_t);
-int dcc_x_file(int ofd, const char *fname, const char *token, size_t *);
-int dcc_r_file_body(int ofd, int ifd, size_t len);
+int dcc_r_file(int ifd, const char *filename, unsigned,
+               enum dcc_compress);
 int dcc_r_fifo(int ifd, const char *fifo_name, size_t len);
+
+int dcc_x_file(int ofd, const char *fname, const char *token, 
+               enum dcc_compress compression);
+
+int dcc_x_file_timed(int ofd, const char *fname, const char *token,
+                     size_t *size_out);
+
+int dcc_r_file_timed(int ifd, const char *fname, unsigned size,
+                     enum dcc_compress);
+
+int dcc_r_token_file(int ifd,
+                     const char *token,
+                     const char *fname,
+                     enum dcc_compress compr);
