@@ -2,7 +2,7 @@
  * 
  * distcc -- A simple distributed compiler system
  *
- * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -85,10 +85,17 @@ static int dcc_x_argv(int fd, char **argv)
 }
 
 
+
+/* Send a request across to the already-open server.
+ *
+ * CPP_PID is the PID of the preprocessor running in the background.
+ * We wait for it to complete before reading its output.
+ */
 static int dcc_send_job(int net_fd,
                         char **argv,
                         pid_t cpp_pid,
                         int *status,
+                        const char *input_fname,
                         const char *cpp_fname,
                         struct dcc_hostdef *host)
 {
@@ -99,7 +106,7 @@ static int dcc_send_job(int net_fd,
         return ret;
 
     if (cpp_pid) {
-        dcc_note_state(STATE_CPP, NULL, NULL);
+        dcc_note_state(DCC_STATE_CPP, NULL, NULL);
         /* Wait for cpp to finish (if not already done), check the
          * result, then send the .i file */
         
@@ -110,7 +117,7 @@ static int dcc_send_job(int net_fd,
          * locally, because we'd presumably get the same result.  Therefore
          * critique the command and log a message and return an indication
          * that compilation is complete. */
-        if (dcc_critique_status(*status, "cpp", dcc_hostdef_local, 0))
+        if (dcc_critique_status(*status, "cpp", input_fname, dcc_hostdef_local, 0))
             return 0;
     }
 
@@ -122,7 +129,7 @@ static int dcc_send_job(int net_fd,
         return EXIT_IO_ERROR;
     }
 
-    dcc_note_state(STATE_SEND, NULL, NULL);
+    dcc_note_state(DCC_STATE_SEND, NULL, NULL);
     
     if ((ret = dcc_x_file(net_fd, cpp_fname, "DOTI", host->compr)))
         return ret;
@@ -144,12 +151,16 @@ static int dcc_send_job(int net_fd,
  * closed while corked.  Unfortunately, this is not a full solution:
  * if distcc crashes or is killed the same situation will pertain.
  *
+ * That's why this is in a separate wrapper function, rather than just
+ * being done in dcc_send_job().
+ *
  * http://marc.theaimsgroup.com/?l=linux-netdev&r=1&b=200209&w=2
  **/
 int dcc_send_job_corked(int net_fd,
 			char **argv,
 			pid_t cpp_pid,
                         int *status,
+                        const char *input_fname,
 			const char *cpp_fname,
                         struct dcc_hostdef *host)
 {
@@ -157,8 +168,8 @@ int dcc_send_job_corked(int net_fd,
 
     tcp_cork_sock(net_fd, 1);
 
-    ret = dcc_send_job(net_fd, argv, cpp_pid, status, cpp_fname,
-                       host);
+    ret = dcc_send_job(net_fd, argv, cpp_pid, status,
+                       input_fname, cpp_fname, host);
     
     tcp_cork_sock(net_fd, 0);
     
@@ -218,7 +229,7 @@ int dcc_retrieve_results(int net_fd,
 
     /* We've started to see the response, so the server is done
      * compiling. */
-    dcc_note_state(STATE_RECEIVE, NULL, NULL);
+    dcc_note_state(DCC_STATE_RECEIVE, NULL, NULL);
 
     if ((ret = dcc_r_cc_status(net_fd, status))
         || (ret = dcc_r_token_int(net_fd, "SERR", &len))
