@@ -2,7 +2,7 @@
  * 
  * distcc -- A simple distributed compiler system
  *
- * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -38,6 +38,7 @@
 #include <string.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <signal.h>
 
 #include "distcc.h"
 #include "trace.h"
@@ -88,14 +89,15 @@ static void dcc_show_usage(void)
 "   --version                  show version and exit\n"
 "\n"
 "Environment variables:\n"
+"   See the manual page for a complete list.\n"
 "   DISTCC_VERBOSE=1           give debug messages\n"
-"   DISTCC_LOG=LOGFILE         send messages here, not stderr\n"
-"   DISTCC_TCP_CORK=0          disable TCP corks\n"
-"   DISTCC_SSH=cmd             command to run to open SSH connections\n"
+"   DISTCC_LOG                 send messages to file, not stderr\n"
+"   DISTCC_SSH                 command to run to open SSH connections\n"
+"   DISTCC_DIR                 directory for host list and locks\n"
 "\n"
 "Server specification:\n"
 "A list of servers is taken from the environment variable $DISTCC_HOSTS, or\n"
-"the file $HOME/.distcc/hosts, or the file %s/distcc/hosts.\n"
+"$DISTCC_DIR/hosts, or ~/.distcc/hosts, or %s/distcc/hosts.\n"
 "Each host can be given in any of these forms, see the manual for details:\n"
 "\n"
 "   localhost                  run in place\n"
@@ -113,12 +115,38 @@ static void dcc_show_usage(void)
 }
 
 
+static RETSIGTYPE dcc_client_signalled (int whichsig)
+{
+    signal(whichsig, SIG_DFL);
+
+#ifdef HAVE_STRSIGNAL
+    rs_log_info("%s", strsignal(whichsig));
+#else
+    rs_log_info("terminated by signal %d", whichsig);
+#endif
+
+    dcc_cleanup_tempfiles();
+
+    raise(whichsig);
+
+}
+
+
+static void dcc_client_catch_signals(void)
+{
+    signal(SIGTERM, &dcc_client_signalled);
+    signal(SIGINT, &dcc_client_signalled);
+    signal(SIGHUP, &dcc_client_signalled);
+}
+
+
+
 /**
  * distcc client entry point.
  *
  * This is typically called by make in place of the real compiler.
  *
- * Performs basic setup and checks for distcc arguments, and then kicks of
+ * Performs basic setup and checks for distcc arguments, and then kicks off
  * dcc_build_somewhere().
  **/
 int main(int argc, char **argv)
@@ -128,6 +156,7 @@ int main(int argc, char **argv)
     char *compiler_name;
     int ret;
 
+    dcc_client_catch_signals();
     atexit(dcc_cleanup_tempfiles);
     atexit(dcc_remove_state_file);
 
@@ -143,24 +172,37 @@ int main(int argc, char **argv)
 
     sg_level = dcc_recursion_safeguard();
 
+    rs_trace("compiler name is \"%s\"", compiler_name);
+
     if (strstr(compiler_name, "distcc") != NULL) {
         /* Either "distcc -c hello.c" or "distcc gcc -c hello.c" */
         if (argc <= 1 || !strcmp(argv[1], "--help")) {
             dcc_show_usage();
-            exit(0);
+            ret = 0;
+            goto out;
         }
         if (!strcmp(argv[1], "--version")) {
             dcc_show_version("distcc");
-            exit(0);
+            ret = 0;
+            goto out;
         }
+        
         dcc_find_compiler(argv, &compiler_args);
+        /* compiler_args is now respectively either "cc -c hello.c" or
+         * "gcc -c hello.c" */
+
+#if 0
+        /* I don't think we need to call this: if we reached this
+         * line, our invocation name is something like 'distcc', and
+         * that's never a problem for masquerading loops. */
         if ((ret = dcc_trim_path(compiler_name)) != 0)
-            dcc_exit(ret);
+            goto out;
+#endif
     } else {
         /* Invoked as "cc -c hello.c", with masqueraded path */
         if ((ret = dcc_support_masquerade(argv, compiler_name,
                                           &tweaked_path)) != 0)
-            dcc_exit(ret);
+            goto out;
         
         dcc_shallowcopy_argv(argv, &compiler_args, 0);
         compiler_args[0] = compiler_name;
@@ -168,8 +210,12 @@ int main(int argc, char **argv)
 
     if (sg_level - tweaked_path > 0) {
         rs_log_crit("distcc seems to have invoked itself recursively!");
-        dcc_exit(EXIT_RECURSION);
+        ret = EXIT_RECURSION;
+        goto out;
     }
 
-    dcc_exit(dcc_build_somewhere_timed(compiler_args, sg_level, &status));
+    ret = dcc_build_somewhere_timed(compiler_args, sg_level, &status);
+
+    out:
+    dcc_exit(ret);
 }
