@@ -34,7 +34,7 @@
 #ifdef UNUSED
 /* nothing */
 #elif defined(__GNUC__)
-#  define UNUSED(x) x __attribute__((unused))
+#  define UNUSED(x) UNUSED_ ## x __attribute__((unused))
 #elif defined(__LCLINT__)
 #  define UNUSED(x) /*@unused@*/ x
 #else				/* !__GNUC__ && !__LCLINT__ */
@@ -121,6 +121,7 @@ int dcc_send_job_corked(int net_fd,
 			char **argv,
 			pid_t cpp_pid,
                         int *status,
+                        const char *,
 			const char *cpp_fname,
                         struct dcc_hostdef *);
 
@@ -142,11 +143,18 @@ int dcc_support_masquerade(char *argv[], char *progname, int *);
 #endif
 
 
-struct sockaddr;
-
-#ifndef HAVE_SOCKADDR_STORAGE
-#  define sockaddr_storage sockaddr_in
-#endif
+/* If --enable-rfc2553 was given, then we will try to enable compile-time IPv6
+ * support.  This means we must have a sockaddr_storage large enough to hold
+ * IPv6 addresses.  If not, we'll just use a plain sockaddr, which is more
+ * likely to compile correctly. */
+#ifdef ENABLE_RFC2553
+#  ifndef HAVE_SOCKADDR_STORAGE
+#    error You can't use RFC2553 because you don't have a sockaddr_storage type
+#  endif /* HAVE_SOCKADDR_STORAGE */
+#  define dcc_sockaddr_storage sockaddr_storage
+#else /* !ENABLE_RFC2553 */
+#  define dcc_sockaddr_storage sockaddr
+#endif /* !ENABLE_RFC2553 */
 
 #ifndef O_BINARY
 #  define O_BINARY 0
@@ -167,3 +175,11 @@ int dcc_compress_file_lzo1x(int in_fd,
                             size_t in_len,
                             char **out_buf,
                             size_t *out_len);
+
+
+/* XXX: Kind of kludgy, we should do dynamic allocation.  But this will do for
+ * now. */
+#ifndef MAXPATHLEN
+#define MAXPATHLEN 4096
+#endif
+
