@@ -93,8 +93,11 @@ int opt_no_prefork = 0;
 const char *arg_pid_file = NULL;
 const char *arg_log_file = NULL;
 
+/* Enumeration values for options that don't have single-letter name.  These
+ * must be numerically above all the ascii letters. */
 enum {
-    opt_log_to_file = 300
+    opt_log_to_file = 300,
+    opt_log_level
 };
 
 
@@ -107,6 +110,7 @@ const struct poptOption options[] = {
     { "lifetime", 0,     POPT_ARG_INT, &opt_lifetime, 0, 0, 0 },
     { "listen", 0,       POPT_ARG_STRING, &opt_listen_addr, 0, 0, 0 },
     { "log-file", 0,     POPT_ARG_STRING, &arg_log_file, 0, 0, 0 },
+    { "log-level", 0,    POPT_ARG_STRING, 0, opt_log_level, 0, 0 },
     { "log-stderr", 0,   POPT_ARG_NONE, &opt_log_stderr, 0, 0, 0 },
     { "nice", 'N',       POPT_ARG_INT,  &opt_niceness,  0, 0, 0 },
     { "no-detach", 0,    POPT_ARG_NONE, &opt_no_detach, 0, 0, 0 },
@@ -143,7 +147,9 @@ static void distccd_show_usage(void)
 "    --listen ADDRESS           IP address to listen on\n"
 "    -a, --allow IP[/BITS]      client address access control\n"
 "  Debug and trace:\n"
-"    --verbose                  include debug messages in log\n"
+"    --log-level=LEVEL          set detail level for log file\n"
+"      levels: critical, error, warning, notice, info, debug\n"
+"    --verbose                  set log level to \"debug\"\n"
 "    --no-detach                don't detach from parent (for daemontools, etc)\n"
 "    --log-file=FILE            send messages here instead of syslog\n"
 "    --log-stderr               send messages to stderr\n"
@@ -208,6 +214,22 @@ int distccd_parse_options(int argc, const char **argv)
             exitcode = EXIT_SUCCESS;
             goto out_exit;
 
+        case opt_log_level:
+            {
+                int level;
+                const char *level_name;
+
+                level_name = poptGetOptArg(po);
+                level = rs_loglevel_from_name(level_name);
+                if (level == -1) {
+                    rs_log_warning("invalid --log-level argument \"%s\"",
+                                   level_name);
+                } else {
+                    rs_trace_set_level(level);
+                }
+            }
+            break;
+
         case 'v':
             rs_trace_set_level(RS_LOG_DEBUG);
             break;
