@@ -67,7 +67,7 @@
  * from inetd. */
 volatile pid_t dcc_master_pid = 0;
 
-static void dcc_daemon_terminate(int);
+static RETSIGTYPE dcc_daemon_terminate(int);
 
 /**
  * Catch all relevant termination signals.  Set up in parent and also
@@ -109,9 +109,14 @@ void dcc_ignore_sighup(void)
  *
  * Some cleanup is done only if we're the master/parent daemon.
  **/
-static void dcc_daemon_terminate(int whichsig)
+static RETSIGTYPE dcc_daemon_terminate(int whichsig)
 {
-    int am_parent = getpid() == dcc_master_pid;
+    int am_parent;
+
+    /* Make sure to remove handler before re-raising signal */
+    signal(whichsig, SIG_DFL);
+
+    am_parent = getpid() == dcc_master_pid;
 
     if (am_parent) {
 #ifdef HAVE_STRSIGNAL
@@ -121,10 +126,6 @@ static void dcc_daemon_terminate(int whichsig)
 #endif
     }
 
-    /* Make sure to remove handler before re-raising signal, or
-     * Valgrind gets its kickers in a knot. */
-    signal(whichsig, SIG_DFL);
-
     dcc_cleanup_tempfiles();
 
     if (am_parent) {
