@@ -2,7 +2,7 @@
  * 
  * distcc -- A simple distributed compiler system
  *
- * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -68,6 +68,12 @@
 #include "hosts.h"
 #include "arg.h"
 
+static void dcc_inside_child(char **argv,
+                             const char *stdin_file,
+                             const char *stdout_file,
+                             const char *stderr_file) NORETURN;
+
+
 static void dcc_execvp(char **argv) NORETURN;
 
 /* You can't have core files on Windows. */
@@ -169,18 +175,16 @@ static void dcc_execvp(char **argv)
  * localhost slots.
  *
  * @param what Type of process to be run here (cpp, cc, ...)
- *
- * @returns Exit code
  **/
-static int dcc_inside_child(char **argv,
-                            const char *stdin_file,
-                            const char *stdout_file,
-                            const char *stderr_file)
+static void dcc_inside_child(char **argv,
+                             const char *stdin_file,
+                             const char *stdout_file,
+                             const char *stderr_file) 
 {
     int ret;
-
-    if ((ret = dcc_ignore_sigpipe(0)))    /* set handler back to default */
-        return ret;
+    
+    if ((ret = dcc_ignore_sigpipe(0)))
+        goto fail;              /* set handler back to default */
 
     /* Ignore failure */
     dcc_increment_safeguard();
@@ -188,9 +192,14 @@ static int dcc_inside_child(char **argv,
     /* do this last, so that any errors from previous operations are
      * visible */
     if ((ret = dcc_redirect_fds(stdin_file, stdout_file, stderr_file)))
-        return ret;
+        goto fail;
     
     dcc_execvp(argv);
+
+    ret = EXIT_DISTCC_FAILED;
+
+    fail:
+    dcc_exit(ret);
 }
 
 
@@ -199,7 +208,10 @@ int dcc_new_pgrp(void)
     /* If we're a session group leader, then we are not able to call
      * setpgid().  However, setsid will implicitly have put us into a new
      * process group, so we don't have to do anything. */
-    if (getpgid(0) == getpid()) {
+
+    /* Does everyone have getpgrp()?  It's in POSIX.1.  We used to call
+     * getpgid(0), but that is not available on BSD/OS. */
+    if (getpgrp() == getpid()) {
         rs_trace("already a process group leader");
         return 0;
     }
@@ -218,9 +230,7 @@ int dcc_new_pgrp(void)
  * Run @p argv in a child asynchronously.
  *
  * stdin, stdout and stderr are redirected as shown, unless those
- * filenames are NULL.
- *
- * Better server-side load limitation must still be organized.
+ * filenames are NULL.  In that case they are left alone.
  *
  * @warning When called on the daemon, where stdin/stdout may refer to random
  * network sockets, all of the standard file descriptors must be redirected!
@@ -239,7 +249,7 @@ int dcc_spawn_child(char **argv, pid_t *pidptr,
         rs_log_error("failed to fork: %s", strerror(errno));
         return EXIT_OUT_OF_MEMORY; /* probably */
     } else if (pid == 0) {
-        exit(dcc_inside_child(argv, stdin_file, stdout_file, stderr_file));
+        dcc_inside_child(argv, stdin_file, stdout_file, stderr_file);
         /* !! NEVER RETURN FROM HERE !! */
     } else {
         *pidptr = pid;
@@ -319,20 +329,17 @@ int dcc_collect_child(const char *what, pid_t pid,
 
 
 /**
- * Analyze and report on a command's exit code.
+ * Analyze and report to the user on a command's exit code.  
  *
  * @param command short human-readable description of the command (perhaps
  * argv[0])
  *
  * @returns 0 if the command succeeded; 128+SIGNAL if it stopped on a
  * signal; otherwise the command's exit code.
- *
- * @todo It would be nice to show the input filename here, however when we're
- * invoked by ccache our input filename may not be meaningful to the user.
- * Therefore I'm going to leave it out for now.
  **/
 int dcc_critique_status(int status,
                         const char *command,
+                        const char *input_fname,
                         struct dcc_hostdef *host,
                         int verbose)
 {
@@ -348,14 +355,14 @@ int dcc_critique_status(int status,
     if (WIFSIGNALED(status)) {
 #ifdef HAVE_STRSIGNAL
         rs_log(logmode,
-               "%s on %s:%s %s",
-               command, host->hostdef_string,
+               "%s %s on %s:%s %s",
+               command, input_fname, host->hostdef_string,
                strsignal(WTERMSIG(status)),
                WCOREDUMP(status) ? " (core dumped)" : "");
 #else
         rs_log(logmode,
-               "%s on %s terminated by signal %d%s",
-               command, host->hostdef_string,
+               "%s %s on %s terminated by signal %d%s",
+               command, input_fname, host->hostdef_string,
                WTERMSIG(status),
                WCOREDUMP(status) ? " (core dumped)" : "");
 #endif
@@ -363,7 +370,7 @@ int dcc_critique_status(int status,
         return 128 + WTERMSIG(status);
     } else if (WEXITSTATUS(status) == 1) {
         /* Normal failure gives exit code 1, so handle that specially */
-        rs_log(logmode, "%s on %s failed", command, host->hostdef_string);
+        rs_log(logmode, "%s %s on %s failed", command, input_fname, host->hostdef_string);
         return WEXITSTATUS(status);
     } else if (WEXITSTATUS(status)) {
         /* This is a tough call; we don't really want to clutter the client's
@@ -371,12 +378,12 @@ int dcc_critique_status(int status,
          * people may find it hard to work things out. */
 
         rs_log(logmode,
-               "%s on %s failed with exit code %d",
-               command, host->hostdef_string, WEXITSTATUS(status));
+               "%s %s on %s failed with exit code %d",
+               command, input_fname, host->hostdef_string, WEXITSTATUS(status));
         return WEXITSTATUS(status);
     } else {
         rs_log(RS_LOG_INFO|RS_LOG_NONAME,
-               "%s on %s completed ok", command, host->hostdef_string);
+               "%s %s on %s completed ok", command, input_fname, host->hostdef_string);
         return 0;
     }
 }
