@@ -22,10 +22,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_STDINT_H
-#  include <stdint.h>
-#endif
-
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
