@@ -2,7 +2,7 @@
  * 
  * distcc -- A simple distributed compiler system
  *
- * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -96,6 +96,7 @@
 #include "hosts.h"
 #include "exitcode.h"
 #include "snprintf.h"
+#include "tempfile.h"
 
 const int dcc_default_port = DISTCC_DEFAULT_PORT;
 
@@ -124,15 +125,14 @@ static char *strndup(const char *src, size_t size)
  * Get a list of hosts to use.
  *
  * Hosts are taken from DISTCC_HOSTS, if that exists.  Otherwise, they are
- * taken from ~/.distcc/hosts, if that exists.  Otherwise, they are taken from
- * ${sysconfdir}/distcc/hosts, if that exists.  Otherwise, we fail.
+ * taken from $DISTCC_DIR/hosts, if that exists.  Otherwise, they are taken
+ * from ${sysconfdir}/distcc/hosts, if that exists.  Otherwise, we fail.
  **/
 int dcc_get_hostlist(struct dcc_hostdef **ret_list,
                      int *ret_nhosts)
 {
     char *env;
-    char *path;
-    char *home;
+    char *path, *top;
     int ret;
 
     if ((env = getenv("DISTCC_HOSTS")) != NULL) {
@@ -140,9 +140,11 @@ int dcc_get_hostlist(struct dcc_hostdef **ret_list,
         return dcc_parse_hosts(env, "$DISTCC_HOSTS", ret_list, ret_nhosts);
     }
 
-    /* ~/.distcc/hosts */
-    if ((home = getenv("HOME")) != NULL) {
-        asprintf(&path, "%s/.distcc/hosts", home);
+    /* $DISTCC_DIR or ~/.distcc */
+    if ((ret = dcc_get_top_dir(&top)) == 0) {
+        /* if we failed to get it, just warn */
+
+        asprintf(&path, "%s/hosts", top);
         if (access(path, R_OK) == 0) {
             ret = dcc_parse_hosts_file(path, ret_list, ret_nhosts);
             free(path);
@@ -337,6 +339,8 @@ static int dcc_parse_localhost(struct dcc_hostdef *hostdef,
 
 
 /**
+ * @p where is the host list, taken either from the environment or file.
+ *
  * @return 0 if parsed successfully; nonzero if there were any errors,
  * or if no hosts were defined.
  **/
@@ -367,13 +371,24 @@ int dcc_parse_hosts(const char *where, const char *source_name,
         const char *token_start;
         int has_at;
         
-        while (isspace(where[0]))
-            where++;            /* skip space */
         if (where[0] == '\0')
             break;              /* end of string */
+        
+        /* skip over comments */
+        if (where[0] == '#') {
+            do
+                where++;
+            while (where[0] != '\n' && where[0] != '\0');
+            continue;
+        }
+
+        if (isspace(where[0])) {
+            where++;            /* skip space */
+            continue;
+        }
 
         token_start = where;
-        token_len = strcspn(where, " \t\n\f");
+        token_len = strcspn(where, " #\t\n\f");
 
         /* Allocate new list item */
         curr = calloc(1, sizeof(struct dcc_hostdef));
@@ -404,8 +419,9 @@ int dcc_parse_hosts(const char *where, const char *source_name,
             
         has_at = (memchr(token_start, '@', token_len) != NULL);
 
-        if (!strncmp(token_start, "localhost", token_len)
-            || !strncmp(token_start, "localhost/", strlen("localhost/"))) {
+        if (!strncmp(token_start, "localhost", 9)
+            && (token_len == 9 || token_start[9] == '/')) {
+            rs_trace("found localhost token \"%.*s\"", token_len, token_start);
             if ((ret = dcc_parse_localhost(curr, token_start)) != 0)
                 return ret;
         } else if (has_at) {
