@@ -24,6 +24,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/socket.h>
 
 #include <signal.h>
 #include <stdio.h>
@@ -48,7 +49,6 @@
 #include "snprintf.h"
 #include "mon.h"
 #include "util.h"
-#include "netutil.h"
 
 
 /**
@@ -58,12 +58,14 @@
  *
  * Every time the client wants an update, it can call dcc_mon_poll(),
  * which returns a newly allocated list of all running processes.
+ *
+ * The list is returned sorted by hostname and then by slot, so tasks
+ * will be more stable from one call to the next.
  **/
 
 
-/* TODO: Shouldn't fail if the directory doesn't exist at the moment.
- *
- * TODO: Don't emit visible warnings for transient failures?
+/* TODO: Shouldn't fail if the directory doesn't exist at the moment
+ * it's called.
  */
 
 
@@ -73,7 +75,7 @@
  * take a very long time, and combined with the check that the process
  * exists we can allow this to be reasonably large.
  */
-const int dcc_state_max_age = 300;
+const int dcc_state_max_age = 60;
 
 
 /**
@@ -114,31 +116,47 @@ static int dcc_mon_kill_old(int fd,
 
 
 static int dcc_mon_read_state(int fd, char *fullpath,
-                              struct dcc_mon_list *lp)
+                              struct dcc_history *lp)
 {
-    int ret;
-    unsigned val;
+    int nread;
 
-    if ((ret = dcc_r_token_int(fd, "STFV", &val)))
-        return ret;
-
-    if (val != 0) {
-        rs_log_warning("bad state file version in %s",
-                       fullpath);
-        return EXIT_PROTOCOL_ERROR;
+    /* Don't use dcc_readx(), because not being able to read it is not
+     * a big deal. */
+    nread = read(fd, lp, sizeof *lp);
+    if (nread == -1) {
+        rs_trace("failed to read state from %s: %s",
+                 fullpath, strerror(errno));
+        return EXIT_IO_ERROR;
+    } else if (nread == 0) {
+        /* empty file; just bad timing. */
+        return EXIT_IO_ERROR;
+    } else if (nread != sizeof *lp) {
+        rs_trace("short read getting state from %s",
+                 fullpath);
+        return EXIT_IO_ERROR;
     }
 
-    if ((ret = dcc_r_token_int(fd, "CPID", &lp->cpid)))
-        return ret;
+    /* sanity-check some fields */
 
-    if ((ret = dcc_r_token_string(fd, "STTE", &lp->state)))
-        return ret;
+    if (lp->magic != DCC_STATE_MAGIC) {
+        rs_log_warning("wrong magic number: %s",
+                       fullpath);
+        return EXIT_IO_ERROR;
+    }
 
-    if ((ret = dcc_r_token_string(fd, "FILE", &lp->file)))
-        return ret;
+    if (lp->struct_size != sizeof (struct dcc_history)) {
+        rs_log_warning("wrong structure size: %s: version mismatch?",
+                       fullpath);
+        return EXIT_IO_ERROR;
+    }
+    
+    lp->file[sizeof lp->file - 1] = '\0';
+    lp->host[sizeof lp->host - 1] = '\0';
+    if (lp->curr_state > DCC_STATE_DONE) {
+        lp->curr_state = DCC_STATE_COMPILE;
+    }
 
-    if ((ret = dcc_r_token_string(fd, "HOST", &lp->host)))
-        return ret;
+    lp->next = 0;
 
     return 0;
 }
@@ -148,9 +166,22 @@ static int dcc_mon_read_state(int fd, char *fullpath,
  * Check that the process named by the file still exists; if not,
  * return EXIT_GONE.
  **/
-static int dcc_mon_check_orphans(struct dcc_mon_list *monl)
+static int dcc_mon_check_orphans(struct dcc_history *monl)
 {
-    return kill(monl->cpid, 0) ? EXIT_GONE : 0;
+    /* signal 0 just checks if it exists */
+    if (!kill(monl->cpid, 0)) {
+        return 0;               /* it's here */
+    } else if (errno == EPERM) {
+        /* It's here, but it's not ours.  Assume it's still a real
+         * distcc process. */
+        return 0;
+    } else if (errno == ESRCH) {
+        return EXIT_GONE;       /* no such pid */
+    } else {
+        rs_log_warning("kill %ld, 0 failed: %s", (long) monl->cpid,
+                       strerror(errno));
+        return EXIT_GONE;
+    }
 }
 
 
@@ -160,43 +191,42 @@ static int dcc_mon_check_orphans(struct dcc_mon_list *monl)
  */
 static int dcc_mon_load_state(int fd,
                               char *fullpath,
-                              struct dcc_mon_list **ppl)
+                              struct dcc_history **ppl)
 {
     int ret;
-    struct dcc_mon_list *tl;
+    struct dcc_history *tl;
 
     tl = calloc(1, sizeof *tl);
     if (!tl)
-        rs_fatal("failed to allocate dcc_mon_list");
+        rs_fatal("failed to allocate dcc_history");
     
     ret = dcc_mon_read_state(fd, fullpath, tl);
-
-    if (ret == 0)
-        ret = dcc_mon_check_orphans(tl);
-
     if (ret) {
-        dcc_mon_list_free(tl);
-    } else {
-        *ppl = tl;
+        dcc_history_free(tl);
+        return ret;
     }
+
+    if (tl->curr_state != DCC_STATE_DONE) {
+        ret = dcc_mon_check_orphans(tl);
+        if (ret)
+            return ret;
+    }
+
+    *ppl = tl;
     
     return ret;
 }
 
 
 /* Free the whole list */
-int dcc_mon_list_free(struct dcc_mon_list *lp)
+int dcc_history_free(struct dcc_history *lp)
 {
-    struct dcc_mon_list *next;
+    struct dcc_history *next;
 
     while (lp) {
         next = lp->next;        /* save from clobbering */
-        
-        free(lp->file);
-        free(lp->host);
-        free(lp->state);
-        free(lp);
 
+        /* nothing dynamically allocated in them anymore */
         lp = next;
     }
     
@@ -211,7 +241,7 @@ int dcc_mon_list_free(struct dcc_mon_list *lp)
  * If a new entry is read, a pointer to it is returned in @p lp.
  **/
 static int dcc_mon_do_file(char *dirname, char *filename,
-                           struct dcc_mon_list **lp)
+                           struct dcc_history **lp)
 {
     int fd;
     char *fullpath;
@@ -258,6 +288,37 @@ static int dcc_mon_do_file(char *dirname, char *filename,
 }
 
 
+/**
+ * Insert @p new into the list at the appropriate sorted position.
+ **/
+static void dcc_mon_insert_sorted(struct dcc_history **list,
+                                  struct dcc_history *new)
+{
+    int s;
+    struct dcc_history *i;
+    
+    for (; (i = *list) != NULL; list = &i->next) {
+        /* Should we go before *list?  If the hostname comes first, or
+         * the name is the same and the slot is lower. */
+        s = strcmp(i->host, new->host);
+
+        if (s > 0) {
+            /* new's host is earlier */
+            break;
+        } else if (s == 0) {
+            /* same host; compare slots */
+            if (new->slot < i->slot)
+                break;
+        } 
+    }
+    
+    /* OK, insert it before the current contents of *list, which may
+     * be NULL */
+    *list = new;
+    new->next = i;
+}
+
+
 /**
  * Read through the state directory and return information about all
  * processes we find there.
@@ -266,13 +327,12 @@ static int dcc_mon_do_file(char *dirname, char *filename,
  * corrupt -- that can easily happen if e.g. a client crashes or is
  * interrupted, or is even just in the middle of writing its file.
  **/
-int dcc_mon_poll(struct dcc_mon_list **p_list)
+int dcc_mon_poll(struct dcc_history **p_list)
 {
     int ret;
     char *dirname;
     DIR *d;
     struct dirent *de;
-    struct dcc_mon_list **ppnext;
 
     *p_list = NULL;
     
@@ -285,17 +345,14 @@ int dcc_mon_poll(struct dcc_mon_list **p_list)
         return ret;
     }
 
-    ppnext = p_list;
     while ((de = readdir(d)) != NULL) {
-        struct dcc_mon_list *pthis;
+        struct dcc_history *pthis;
         if (dcc_mon_do_file(dirname, de->d_name, &pthis) == 0
             && pthis) {
             /* We can succeed without getting a new entry back, but it
-             * turns out that this time we did get one.  So store a
-             * pointer to it onto the list, and prepare to store the
-             * next pointer into this node's link slot. */
-            *ppnext = pthis;
-            ppnext = &(pthis->next);
+             * turns out that this time we did get one.  So insert it
+             * intot he right point on the list. */
+            dcc_mon_insert_sorted(p_list, pthis);
         }
     }
 
