@@ -124,13 +124,14 @@ static int dcc_preforked_child(int listen_fd)
     
     for (ireq = 0; ireq < child_lifetime; ireq++) {
         int acc_fd;
-        struct sockaddr cli_addr;
+        struct dcc_sockaddr_storage cli_addr;
         socklen_t cli_len;
         
         cli_len = sizeof cli_addr;
 
         do {
-            acc_fd = accept(listen_fd, &cli_addr, &cli_len);
+            acc_fd = accept(listen_fd, (struct sockaddr *) &cli_addr,
+                            &cli_len);
         } while (acc_fd == -1 && errno == EINTR);
 
         if (acc_fd == -1) {
@@ -138,7 +139,8 @@ static int dcc_preforked_child(int listen_fd)
             dcc_exit(EXIT_CONNECT_FAILED);
         }
         
-        dcc_service_job(acc_fd, acc_fd, &cli_addr, cli_len);
+        dcc_service_job(acc_fd, acc_fd,
+                        (struct sockaddr *) &cli_addr, cli_len);
 
         dcc_close(acc_fd);
     }
