@@ -63,7 +63,8 @@
 /* If you don't have it, just use dcc_pump_readwrite */
 
 /**
- * Map FreeBSD and Linux into something like the Linux interface.
+ * sys_sendfile maps all the different implementations of sendfile() into
+ * something like the Linux interface.
  *
  * Our sockets are never non-blocking, so that seems to me to say that
  * the kernel will never return EAGAIN -- we will always either send
@@ -75,10 +76,9 @@
  * @param offset offset in input to start writing; updated on return
  * to reflect the number of bytes sent.
  *
- * @return number of bytes sent; -1 otherwise, with errno set.  (As
- * for Linux)
- *
- * @sa http://www.freebsd.org/cgi/man.cgi?query=sendfile&sektion=2&apropos=0&manpath=FreeBSD+5.0-current
+ * sys_sendfile returns the number of bytes sent, if transmission succeeded.
+ * If there was an error, it returns -1 with errno set.  It should never
+ * return 0.
  **/
 
 
@@ -94,9 +94,17 @@ static ssize_t sys_sendfile(int ofd, int ifd, off_t *offset, size_t size)
     if (ret == -1) {
         /* http://cvs.apache.org/viewcvs.cgi/apr/network_io/unix/sendrecv.c?rev=1.95&content-type=text/vnd.viewcvs-markup */
         if (errno == EAGAIN) {
-            *offset += sent_bytes;
-            return sent_bytes;
+            if (sent_bytes == 0) {
+                /* Didn't send anything. Return error with errno == EAGAIN. */
+                return -1;
+            } else {
+                /* We sent some bytes, but they we would block.  Treat this as
+                 * success for now. */
+                *offset += sent_bytes;
+                return sent_bytes;
+            }
         } else {
+            /* some other error */
             return -1;
         }
     } else if (ret == 0) {
