@@ -1,7 +1,6 @@
 /* -*- c-file-style: "java"; indent-tabs-mode: nil; fill-column: 78; -*-
  * 
  * distcc -- A simple distributed compiler system
- * $Header: /data/cvs/distcc/src/setuid.h,v 1.1 2003/01/27 11:57:25 mbp Exp $
  *
  * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
  *
@@ -23,4 +22,4 @@
 
 /* setuid.c */
 int dcc_discard_root(void);
-extern char *opt_user;
+extern const char *opt_user;
