@@ -2,7 +2,7 @@
  * 
  * distcc -- A simple distributed compiler system
  *
- * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -72,24 +72,17 @@
 #include "netutil.h"
 #include "dopt.h"
 
-/**
+/*
  * Listen on a predetermined address (often the passive address).  The way in
  * which we get the address depends on the resolver API in use.
  **/
-static int dcc_listen_by_addr(int *listen_fd,
+static int dcc_listen_by_addr(int fd,
                               struct sockaddr *sa,
                               size_t salen)
 {
     int one = 1;
-    int fd;
     char *sa_buf = NULL;
 
-    fd = socket(sa->sa_family, SOCK_STREAM, 0);
-    if (fd == -1) {
-	rs_log_error("socket creation failed: %s", strerror(errno));
-	return EXIT_BIND_FAILED;
-    }
-
     setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (char *) &one, sizeof(one));
 
     dcc_sockaddr_to_string(sa, salen, &sa_buf);
@@ -111,41 +104,22 @@ static int dcc_listen_by_addr(int *listen_fd,
         close(fd);
         return EXIT_BIND_FAILED;
     }
-    
-    *listen_fd = fd;
-    return 0;
 
+    return 0;
 }
 
 
 
 #if defined(ENABLE_RFC2553)
-
-/**
- * In general if there are two equally good addresses, we'd prefer to use the
- * IPv4 one because that is least surprising for current users.  So this
- * function look through a lists for an entry that is on IPv4.  
- **/
-static int dcc_prefer_ipv4(struct addrinfo *list, struct addrinfo **best)
-{
-    *best = list;
-    
-    for (;list; list = list->ai_next) {
-        if (list->ai_family == AF_INET)
-            *best = list;
-    }
-
-    return 0;
-}
-
-
-/* This version uses getaddrinfo */
-int dcc_socket_listen(int port, int *listen_fd, const char *listen_addr)
+/* This version uses getaddrinfo.  It will probably use IPv6 if that's
+ * supported by your configuration, kernel, and library. */
+int dcc_socket_listen(int port, int *fd_out, const char *listen_addr)
 {
     char portname[20];
     struct addrinfo hints;
-    struct addrinfo *res, *best;
+    struct addrinfo *res, *ai;
     int error;
+    int ret;
 
     /* getaddrinfo() ought to check for this, but some versions do not.
      * (Debian Bug#192876.) */
@@ -173,11 +147,27 @@ int dcc_socket_listen(int port, int *listen_fd, const char *listen_addr)
         return EXIT_BIND_FAILED;
     }
 
-    dcc_prefer_ipv4(res, &best);
+    /* The first sockaddr returned will typically be an IPv6 socket.  Some
+     * kernels might not support that. */
+    for (ai = res; ai; ai=ai->ai_next) {
+        int af = ai->ai_addr->sa_family;
+        if ((*fd_out = socket(af, SOCK_STREAM, 0)) == -1) {
+            if (errno == EAFNOSUPPORT) {
+                rs_log_notice("socket address family %d not supported", af);
+                continue;
+            } else {
+                rs_log_error("socket creation failed: %s", strerror(errno));
+                return EXIT_BIND_FAILED;
+            }
+        } else {
+            ret = dcc_listen_by_addr(*fd_out, res->ai_addr, res->ai_addrlen);
+            freeaddrinfo(res);
+            return ret;
+        }
+    }
 
-    return dcc_listen_by_addr(listen_fd, best->ai_addr, best->ai_addrlen);
-
-    /* FIXME: This leaks the res list. */
+    rs_log_error("failed to find any supported socket family");
+    return EXIT_BIND_FAILED;
 }
 
 #else /* ndef ENABLE_RFC2553 */
@@ -207,7 +197,13 @@ int dcc_socket_listen(int port, int *listen_fd, const char *listen_addr)
         sock.sin_addr.s_addr = INADDR_ANY;
     }
 
-    return dcc_listen_by_addr(listen_fd, (struct sockaddr *) &sock, sizeof sock);
+    if ((*listen_fd = socket(PF_INET, SOCK_STREAM, 0)) == -1) {
+	rs_log_error("socket creation failed: %s", strerror(errno));
+	return EXIT_BIND_FAILED;
+    }
+
+    return dcc_listen_by_addr(*listen_fd, (struct sockaddr *) &sock,
+                              sizeof sock);
 }
 #endif  /* ndef ENABLE_RFC2553 */
 
@@ -225,9 +221,12 @@ int is_a_socket(int fd)
 
 /**
  * Log client IP address and perform access control checks.
+ *
+ * Note that PSA may be NULL if the sockaddr is unknown.
  **/
 int dcc_check_client(struct sockaddr *psa,
-                     int salen)
+                     int salen,
+                     struct dcc_allow_list *allowed)
 {
     char *client_ip;
     struct dcc_allow_list *l;
@@ -239,25 +238,47 @@ int dcc_check_client(struct sockaddr *psa,
     rs_log_info("connection from %s", client_ip);
     free(client_ip);
 
-    /* if there are any access entries, then we must match one */
-    if (opt_allowed) {
-        for (l = opt_allowed; l; l = l->next) {
-            if (psa->sa_family == AF_INET) {
-                /* At the moment, access control is only supported for
-                   IPv4 */
-                in_addr_t cli_inaddr;
-                cli_inaddr = ((struct sockaddr_in *) psa)->sin_addr.s_addr;
+    if (!psa)
+        /* if no sockaddr, must be a pipe or something. */
+        return 0;
+
+    if (!allowed)
+        /* if no ACL, default open */
+        return 0;
+    
+    for (l = allowed; l; l = l->next) {
+        if (psa->sa_family == AF_INET) {
+            in_addr_t cli_inaddr;
+            cli_inaddr = ((struct sockaddr_in *) psa)->sin_addr.s_addr;
                 
-                if ((ret = dcc_check_address(cli_inaddr,
-                                             l->addr, l->mask)) == 0)
+            if ((ret = dcc_check_address(cli_inaddr, l->addr, l->mask)) == 0)
+                break;
+#ifdef ENABLE_RFC2553
+        } else if (psa->sa_family == AF_INET6) {
+            const struct sockaddr_in6 *sa6 = (const struct sockaddr_in6 *) psa;
+            const struct in6_addr *a6 = &sa6->sin6_addr;
+            const in_addr_t *a4;
+            
+            if (IN6_IS_ADDR_V4MAPPED(a6) || IN6_IS_ADDR_V4COMPAT(a6)) {
+                a4 = (const in_addr_t *) &a6->s6_addr[12];
+                if ((ret = dcc_check_address(*a4, l->addr, l->mask)) == 0)
                     break;
+            } else {
+                rs_log_notice("ipv6 ACLs not implemented");
+                return EXIT_ACCESS_DENIED;
             }
-        }
-        if (ret != 0) {
-            rs_log_error("connection denied by access list");
-            return ret;
+#endif
+        } else {
+            ret = EXIT_ACCESS_DENIED;
+            rs_log_notice("access denied from unknown address family %d",
+                          psa->sa_family);
+            break;
         }
     }
 
-    return 0;
+    if (ret != 0) {
+        rs_log_error("connection denied by access list");
+    }
+    return ret;
 }
+
