@@ -2,7 +2,7 @@
  * 
  * distcc -- A simple distributed compiler system
  *
- * Copyright (C) 2002, 2003 by Martin Pool <mbp@samba.org>
+ * Copyright (C) 2002, 2003, 2004 by Martin Pool <mbp@samba.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -108,7 +108,7 @@ static int dcc_get_tmp_top(const char **p_ret)
  **/
 int dcc_mkdir(const char *path)
 {
-    if ((mkdir(path, 0755) == -1) && (errno != EEXIST)) {
+    if ((mkdir(path, 0777) == -1) && (errno != EEXIST)) {
         rs_log_error("mkdir %s failed: %s", path, strerror(errno));
         return EXIT_IO_ERROR;
     }
@@ -121,7 +121,7 @@ int dcc_mkdir(const char *path)
  * Return a static string holding DISTCC_DIR, or ~/.distcc.
  * The directory is created if it does not exist.
  **/
-static int dcc_get_top_dir(char **path_ret)
+int dcc_get_top_dir(char **path_ret)
 {
     char *env;
     static char *cached;
@@ -142,7 +142,7 @@ static int dcc_get_top_dir(char **path_ret)
     }
 
     if ((env = getenv("HOME")) == NULL) {
-        rs_log_error("HOME is not set; can't find distcc directory");
+        rs_log_warning("HOME is not set; can't find distcc directory");
         return EXIT_BAD_ARGUMENTS;
     }
 
@@ -176,7 +176,6 @@ static int dcc_get_subdir(const char *name,
         return EXIT_OUT_OF_MEMORY;
     }
 
-
     return dcc_mkdir(*dir_ret);
 }
 
@@ -237,6 +236,11 @@ int dcc_make_tmpnam(const char *prefix,
     if ((ret = dcc_get_tmp_top(&tempdir)))
         return ret;
 
+    if (access(tempdir, W_OK|X_OK) == -1) {
+        rs_log_error("can't use TMPDIR \"%s\": %s", tempdir, strerror(errno));
+        return EXIT_IO_ERROR;
+    }
+
     random_bits = (unsigned long) getpid() << 16;
 
 # if HAVE_GETTIMEOFDAY
