@@ -153,6 +153,36 @@ rs_trace_set_level(rs_loglevel level)
 }
 
 
+
+/**
+ * Work out a log level from a string name. 
+ *
+ * Returns -1 for invalid names.
+ */
+int
+rs_loglevel_from_name(const char *name)
+{
+    if (!strcmp(name, "emerg") || !strcmp(name, "emergency"))
+        return RS_LOG_EMERG;
+    else if (!strcmp(name, "alert"))
+        return RS_LOG_ALERT;
+    else if (!strcmp(name, "critical") || !strcmp(name, "crit"))
+        return RS_LOG_CRIT;
+    else if (!strcmp(name, "error") || !strcmp(name, "err"))
+        return RS_LOG_ERR;
+    else if (!strcmp(name, "warning") || !strcmp(name, "warn"))
+        return RS_LOG_WARNING;
+    else if (!strcmp(name, "notice") || !strcmp(name, "note"))
+        return RS_LOG_NOTICE;
+    else if (!strcmp(name, "info"))
+        return RS_LOG_INFO;
+    else if (!strcmp(name, "debug"))
+        return RS_LOG_DEBUG;
+    
+    return -1;
+}
+
+
 /**
  * If you don't initialize a logger before first logging, then we
  * write to stderr by default.
