@@ -122,9 +122,9 @@ static void dcc_lock_pause(void)
      * really necessary, and also by making jobs complete very-out-of-order is
      * more likely to find Makefile bugs. */
 
-    int pause_time = 2;
+    int pause_time = 1;
 
-    dcc_note_state(STATE_BLOCKED, "", "");
+    dcc_note_state(DCC_STATE_BLOCKED, NULL, NULL);
     
     rs_trace("nothing available, sleeping %ds...", pause_time);
 
@@ -158,14 +158,8 @@ static int dcc_lock_one(struct dcc_hostdef *hostlist,
                 ret = dcc_lock_host("cpu", h, i_cpu, 0, cpu_lock_fd);
                 
                 if (ret == 0) {
-                    /* If this is localhost, there is no transmission phase
-                     * and we don't take a lock */
-                    if (h->mode == DCC_MODE_LOCAL) {
-                        *buildhost = h;
-                        return 0;
-                    }
-
                     *buildhost = h;
+                    dcc_note_state_slot(i_cpu);
                     return 0;
                 } else if (ret == EXIT_BUSY) {
                     continue;
